/*
  
  VIDEO SANS-VIDEO GAME MINI
  pomp.com/2014/02/video-sans-video-game-mini/
  
  Michael Newman
  Pomp Productions
  pomp.com
  
  Version 1.0
  2014-02-05
  
  ARDUINO CONFIGURATION:
  
    LCD PIN: 2
    
    CONTINUOUS ROTATION SERVO PINS:
      Scroll: 3
      Ship Y: 5
      Ship X: 6
      
    JOYSTICK PINS:
      Button: 8
      Up:     9 
      Right:  10
      Down:   11
      Left:   12 
      
    QRE1113 PINS:
      Player's Ship: A1
      Level Tracker: A0 

  NOTE: Servos require alternate power source. Do not connect servos directly to Arduino pins. 
  
*/
 
#include <EEPROM.h>
#include <Servo.h> 
#include <SoftwareSerial.h>

// GAME STATES
boolean DEBUG     = true;   // OUTPUTS DATA TO SERIAL MONITOR
boolean SERVO_RVS = false;  // REVERSES SERVO DIRECTION
boolean MENU      = false;
boolean HOMING    = false;
boolean HOME_S    = false;
boolean REWIND    = false;
boolean REWIND_S  = false;
boolean START_S   = false;
boolean PLAYING   = false;
boolean READY     = false;
boolean COMPLETE  = false;
boolean CALIBRATE = false;
boolean FREE_RUN  = false;
boolean SCROLL    = false;

// MENU
int MENU_STATE   = 0;
const int MENU_L = 5;
const char* MENUS[] = {
  ">>>> VIDEO <<<< SANS-VIDEO GAME",
  "Press button to START GAME",
  "Press button to CALIBRATE IR",
  "Press button to RUN X/Y",
  "Press button to SCROLL",
  "Press button to RESET GAME"
};

// CONFIGURE SERVOS
const int servo_s   = 3;
const int servo_y   = 5;
const int servo_x   = 6;
  int servo_cw  = 180;
  int servo_ccw = 0;     
  
Servo axis_s, axis_x, axis_y;

boolean game_ctl = false;
boolean move_dwn = false;
boolean move_up  = false;
boolean move_rvs = false;
boolean move_fwd = false;

// CONFIGURE JOYSTICK
const int JS_UP     = 9;
const int JS_RIGHT  = 10;
const int JS_DOWN   = 11;
const int JS_LEFT   = 12;
const int JS_BUTTON = 8;   

long DEBOUNCE_DELAY = 10;   

// CONFIGURE SENSORS
const int TRACKER   = A0;
const int TRACKER_E = 0;
  int TRACKER_D     = 970;
  int TRACKER_L     = 0; 
const int SENSOR    = A1;
const int SENSOR_E = 1;
  int SENSOR_D      = 900; 
  int SENSOR_L      = 0; 

long START_DELAY    = 3000;
long SENSOR_DELAY   = 1000;
long SENSOR_TIME    = 0;

// CONFIGURE LCD
const int LCD_DELAY = 2;
const int LCD_DISP  = 2;
SoftwareSerial LCD  = SoftwareSerial(0, LCD_DISP);


void setup () {

  pinMode(JS_UP,     INPUT);     
  pinMode(JS_RIGHT,  INPUT);     
  pinMode(JS_DOWN,   INPUT);     
  pinMode(JS_LEFT,   INPUT);   
  pinMode(JS_BUTTON, INPUT);     
  
  digitalWrite(JS_UP,     HIGH);
  digitalWrite(JS_RIGHT,  HIGH);
  digitalWrite(JS_DOWN,   HIGH);
  digitalWrite(JS_LEFT,   HIGH);
  digitalWrite(JS_BUTTON, HIGH);
  
  // IF SERVO_RVS - FLIP SERVO ROTATION
  if (SERVO_RVS) {
    servo_cw  = 0;
    servo_ccw = 180;
  }
  
  // LOAD QRE VALUES FROM EEPROM
  TRACKER_L = (qre_eeprom_read(TRACKER_E) > 0) ? qre_eeprom_read(TRACKER_E) : TRACKER_D;
  SENSOR_L  = (qre_eeprom_read(SENSOR_E) > 0)  ? qre_eeprom_read(SENSOR_E)  : SENSOR_D;
  
  Serial.begin(9600);
  pinMode(LCD_DISP,  OUTPUT);
  LCD.begin(9600);
  clearLCD();
  backlightOn();
  
  init_game();
}


void init_game () {
  
  MENU     = false;
  HOMING   = false;
  HOME_S   = false;
  REWIND   = false;
  REWIND_S = false;
  START_S  = false;
  PLAYING  = false;
  READY    = false;
  COMPLETE = false;
  CALIBRATE = false;
  FREE_RUN = false;
  SCROLL   = false;
  
  servos_all_off();
  menu_display();
}

void menu_display () {

  MENU = true;
  MENU_STATE = 0;
  lcd_display(MENUS[MENU_STATE]);
}

void homing_loop () {
  
  if (!HOME_S) {
    
    HOME_S = true;
    
    lcd_display("HOMING SYSTEM");
    delay(1000);
    
    REWIND = true;
    axis_s.attach(servo_s);
    axis_s.write(servo_ccw);
    
    SENSOR_TIME = millis();
  }
  
  if (REWIND) {
    if ((millis() - SENSOR_TIME) > SENSOR_DELAY) {
      if (analogRead(TRACKER) > TRACKER_L) {
        
        HOME_S = true;
        REWIND = false;    
        
        READY  = true;
        HOMING = false;
        
        servos_all_off();
        
        menu_display(); 
      }
    }
  }
}

void play_loop () {
  
  if (START_S) {
  
     START_S = false;
     READY   = false;
     
     lcd_display("READY?");
     	delay(1000);
     		lcd_display("SET...");
     			delay(1000);
     				lcd_display("GO!");
     					delay(500);
     clearLCD();
     
     axis_s.attach(servo_s);
     axis_s.write(servo_cw);
     
     SENSOR_TIME = millis();
  }
  
  // SENSORS
  if ((millis() - SENSOR_TIME) > START_DELAY) {
    
    int tracker  = analogRead(TRACKER);
    int sensor   = analogRead(SENSOR);
    
    if (tracker > TRACKER_L) game_over(true);
    if (sensor > SENSOR_L) game_over(false);
    
    if (DEBUG) {
      Serial.print("TRACKER: ");
      Serial.print(analogRead(TRACKER));
      Serial.print(" SENSOR: ");
      Serial.println(analogRead(SENSOR)); 
    }
  }
}

void game_over (boolean win) {
  
  PLAYING   = false;
  COMPLETE  = true;
  
  servos_all_off();
  
  (win) ? lcd_display("WINNER!") : lcd_display("CRASH.");
  delay(1000);
}

void servos_all_off () {
  
  axis_s.write(90);
  axis_s.detach();
  
  axis_x.write(90);
  axis_x.detach();
  
  axis_y.write(90);
  axis_y.detach();
}

void loop () {
  
  // JOYSTICK BUTTON
  if (digitalRead(JS_BUTTON) == LOW) {     
    if (!game_ctl) {
      game_ctl = true;
      joystick_event(JS_BUTTON, game_ctl);
      delay(DEBOUNCE_DELAY);
    }
  } else {
    if (game_ctl) {
      game_ctl = false;
      joystick_event(JS_BUTTON, game_ctl);
      delay(DEBOUNCE_DELAY);
    }
  }   

  // JOYSTICK UP
  if (digitalRead(JS_UP) == LOW) {     
    if (!move_up) {
      move_up = true;
      joystick_event(JS_UP, move_up);
      delay(DEBOUNCE_DELAY);
    }
  } else {
    if (move_up) {
      move_up = false;
      joystick_event(JS_UP, move_up);
      delay(DEBOUNCE_DELAY);
    }
  }   
  
  // JOYSTICK RIGHT
  if (digitalRead(JS_RIGHT) == LOW) {     
    if (!move_fwd) {
      move_fwd = true;
      joystick_event(JS_RIGHT, move_fwd);
      delay(DEBOUNCE_DELAY);
    }
  } else {
    if (move_fwd) {
      move_fwd = false;
      joystick_event(JS_RIGHT, move_fwd);
      delay(DEBOUNCE_DELAY);
    }
  }  
  
  // JOYSTICK DOWN
  if (digitalRead(JS_DOWN) == LOW) {     
    if (!move_dwn) {
      move_dwn = true;
      joystick_event(JS_DOWN, move_dwn);
      delay(DEBOUNCE_DELAY);
    }
  } else {
    if (move_dwn) {
      move_dwn = false;
      joystick_event(JS_DOWN, move_dwn);
      delay(DEBOUNCE_DELAY);
    }
  }
  
  // JOYSTICK LEFT
  if (digitalRead(JS_LEFT) == LOW) {     
    if (!move_rvs) {
      move_rvs = true;
      joystick_event(JS_LEFT, move_rvs);
      delay(DEBOUNCE_DELAY);
    }
  } else {
    if (move_rvs) {
      move_rvs = false;
      joystick_event(JS_LEFT, move_rvs);
      delay(DEBOUNCE_DELAY);
    }
  }
  
  if (HOMING)   homing_loop();
  if (PLAYING)  play_loop();
  
  if (FREE_RUN || REWIND || SCROLL) {
    Serial.print("TRACKER: ");
    Serial.print(analogRead(TRACKER));
    Serial.print(" SENSOR: ");
    Serial.println(analogRead(SENSOR));
  }
}

void joystick_event (int event, boolean value) {

	if (MENU) {
	  if (event == JS_DOWN && value == true) {
	    MENU_STATE = (MENU_STATE < MENU_L) ? MENU_STATE +1 : 0;
			lcd_display(MENUS[MENU_STATE]);  
      	} else if (event == JS_UP && value == true) {
      		MENU_STATE = (MENU_STATE > 0) ? MENU_STATE -1 : MENU_L;
			lcd_display(MENUS[MENU_STATE]); 
      	} else if (event == JS_BUTTON && value == true) {
      		switch (MENU_STATE) {
				case 1:
					MENU     = false;
					START_S  = true;
					(READY) ? PLAYING = true : HOMING = true;  
					return;
                                        break;
				case 2:
					MENU        = false;
                                        CALIBRATE   = true;
                                        return;
					break;
				case 3:
					MENU     = false;
					FREE_RUN = true;
                                        lcd_display("X/Y Free Run    Button Exits"); 
                                        return;
					break;
                                case 4:
					MENU     = false;
					SCROLL   = true;
                                        lcd_display("Scroll Fwd/Rvs  Button Exits"); 
                                        return;
					break;
                                case 5:
					MENU     = false;
					init_game();
                                        return;
					break;
				default:
				break;
			}
			delay(250);
      	}
	}
	
	if (CALIBRATE) {
	  if (event == JS_LEFT || event == JS_RIGHT && value == true) {
	    if (event == JS_LEFT && SENSOR_L > 0) SENSOR_L--; 
            if (event == JS_RIGHT && SENSOR_L < 1023) SENSOR_L++;
      	  } else if (event == JS_DOWN || event == JS_UP && value == true) {
      	    if (event == JS_DOWN && TRACKER_L > 0) TRACKER_L--;
	    if (event == JS_UP && TRACKER_L < 1023) TRACKER_L++;
      	  } else if (event == JS_BUTTON && value == true) {
      	    
            qre_eeprom_write(TRACKER_L, TRACKER_E);
            qre_eeprom_write(SENSOR_L, SENSOR_E);
            
            CALIBRATE = false;
	    menu_display();
            return;
      	  }
      	
      	  clearLCD();
	  lcdPosition(0,0);
	  LCD.print("Tracker: ");
	  LCD.print(TRACKER_L);
	  lcdPosition(1,0);
	  LCD.print("Sensor: ");
	  LCD.print(SENSOR_L);

          delay(100);
	}
	
	if (PLAYING || HOMING || FREE_RUN) {
	  if (event == JS_DOWN || event == JS_UP) {
	    if (value == false) {
		axis_y.write(90);
      		axis_y.detach();
      	    } else {
      	        axis_y.attach(servo_y);
      		axis_y.write((event == JS_DOWN) ? servo_ccw : servo_cw);
      	    }
      	  } else if (event == JS_LEFT || event == JS_RIGHT) {
      	    if (value == false) {
	      axis_x.write(90);
      	      axis_x.detach();
      	    } else {
      	      axis_x.attach(servo_x);
      	      axis_x.write((event == JS_LEFT) ? servo_ccw : servo_cw);
      	    }
      	  } else {
      	    // BUTTON
      	  }
	}
	
        if (SCROLL) {
	  if (event == JS_DOWN || event == JS_UP) {
	    if (value == false) {
		axis_y.write(90);
      		axis_y.detach();
      	    } else {
      	        axis_y.attach(servo_y);
      		axis_y.write((event == JS_DOWN) ? servo_ccw : servo_cw);
      	    }
      	  } else if (event == JS_LEFT || event == JS_RIGHT) {
      	    if (value == false) {
	      axis_s.write(90);
      	      axis_s.detach();
      	    } else {
      	      axis_s.attach(servo_s);
      	      axis_s.write((event == JS_LEFT) ? servo_ccw : servo_cw);
      	    }
      	  } else {
      	    // BUTTON
      	  }
	}

        if (COMPLETE) {
	  if (event == JS_DOWN || event == JS_UP) {
            // UP/DOWN
      	  } else if (event == JS_LEFT || event == JS_RIGHT) {
            // LEFT/RIGHT
      	  } else {
      	    // BUTTON
      	  }
	}

        if (COMPLETE && event == JS_BUTTON && value == true) init_game();
	if (FREE_RUN && event == JS_BUTTON && value == true) init_game();
        if (SCROLL && event == JS_BUTTON && value == true) init_game();
}

// EEPROM FUNCTIONS
void qre_eeprom_write (int value, int location) {
  
  int count = 5;
  int length = 4;
  int write_pos = location *count;
  int write_stop = write_pos + length;
  
  int divide = value /length;
  int modulo = value %length;
  
  while (write_pos < write_stop) {
    EEPROM.write(write_pos, divide);
    write_pos++;
  }
  
  EEPROM.write(write_pos, modulo);
}

int qre_eeprom_read (int location) {
  
  int count = 5;
  int read_pos = location *count;
  int read_stop  = read_pos + count;
  
  int value = 0;
  
  while (read_pos < read_stop) {
    value += EEPROM.read(read_pos);
    read_pos++;
  } 
  
  return value;
}

// LCD FUNCTIONS
void lcd_display (String text) {
  
  clearLCD();
  
  lcdPosition(0,0);
  LCD.print(text.substring(0, 15));
  
  if (text.length() > 16) {
    lcdPosition(1,0);
    LCD.print(text.substring(16));
  }
}

void lcdPosition (int row, int col) {

  LCD.write(0xFE);   
  LCD.write((col + row*64 + 128));   
  delay(LCD_DELAY);
}

void clearLCD () {

  LCD.write(0xFE);   
  LCD.write(0x01);  
  delay(LCD_DELAY);
  lcdPosition(0,0);
}

void backlightOn () {  

  LCD.write(0x7C);  
  LCD.write(157);    
  delay(LCD_DELAY);
}

void backlightOff () {  

  LCD.write(0x7C);   
  LCD.write(128);     
  delay(LCD_DELAY);
}

void serCommand () {   

  LCD.write(0xFE);
}



